from .client import RestClient
from .recognition import RecognitionClient
from .tagging import GenericTaggingClient, FashionTaggingClient
from .colors import DominantColorProductClient, DominantColorGenericClient
from .search import SmartSearchClient
