# General
RECORDS = 'records'
FILE = '_file'
URL = '_url'
BASE64 = '_base64'
IMG_DATA = '_img_data'
RESULT_OK = {'status': 'OK'}

# Recognition/Vize related
RESULTS = 'results'
ID = 'id'
NAME = 'name'
TASK = 'task'
TASKS_COUNT = 'tasksCount'
